package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.CorridaTramoDAO;
import com.rjconsultores.ventaboletos.entidad.CorridaTramo;
import com.rjconsultores.ventaboletos.entidad.RutaCombinacion;
import java.math.BigDecimal;
import org.hibernate.Query;

@Repository("corridaTramoDAO")
public class CorridaTramoHibernateDAO extends GenericHibernateDAO<CorridaTramo, Integer> implements CorridaTramoDAO {

    @Autowired
    public CorridaTramoHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
        setSessionFactory(factory);
    }

    @Override
    public List<CorridaTramo> obtenerTramoTarjetaEmitida() {
        Criteria c = getSession().createCriteria(getPersistentClass());
        c.add(Restrictions.isNotNull("fechortarjetaviaje"));
        c.add(Restrictions.isNull("indmanttarjeta"));
        c.add(Restrictions.eq("numsecuencia", (short) 1));

        c.add(Restrictions.eq("activo", Boolean.TRUE));

        return c.list();
    }

    public List<CorridaTramo> buscarTarjetaMesmaCorrida(CorridaTramo ct) {
        Criteria c = getSession().createCriteria(getPersistentClass());
        c.add(Restrictions.eq("activo", Boolean.TRUE));
        c.add(Restrictions.eq("corrida", ct.getCorrida()));

        return c.list();
    }

    public BigDecimal buscarSumTarjetaMesmaCorrida(CorridaTramo ct) {

        String hql = " select sum(t.kmReal) from CorridaTramo ct, Tramo t "
                + " where ct.tramo.tramoId = t.tramoId and ct.activo = 1 "
                + " and ct.corrida.id.corridaId = :id "
                + " and ct.corrida.id.feccorrida = :data "
                + " group by ct.corrida.id.corridaId,ct.corrida.id.feccorrida ";

        Query sq = getSession().createQuery(hql);
        sq.setParameter("id", ct.getCorrida().getId().getCorridaId());
        sq.setDate("data", ct.getCorrida().getId().getFeccorrida());

        BigDecimal km = (BigDecimal) sq.uniqueResult();

        return km;
    }
}
