/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.EsquemaCorridaDAO;
import com.rjconsultores.ventaboletos.entidad.ClaseServicio;
import com.rjconsultores.ventaboletos.entidad.EsquemaCorrida;
import com.rjconsultores.ventaboletos.entidad.EsquemaOperacional;
import com.rjconsultores.ventaboletos.entidad.Marca;
import com.rjconsultores.ventaboletos.entidad.RolOperativo;
import com.rjconsultores.ventaboletos.entidad.Ruta;
import com.rjconsultores.ventaboletos.entidad.Tramo;

/**
 * 
 * @author Rafius
 */
@Repository("esquemaCorridaDAO")
public class EsquemaCorridaHibernateDAO extends GenericHibernateDAO<EsquemaCorrida, Integer> implements EsquemaCorridaDAO {

	@Autowired
	public EsquemaCorridaHibernateDAO(
			@Qualifier("sessionFactory") SessionFactory factory) {
		setSessionFactory(factory);
	}

	@Override
	public List<EsquemaCorrida> obtenerTodos() {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		return c.list();
	}

	public List<EsquemaCorrida> buscarPorEsquemaOperacional(
			EsquemaOperacional eo) {
		Criteria c = getSession().createCriteria(getPersistentClass());

		c.add(Restrictions.eq("esquemaOperacional", eo));
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		return c.list();
	}

	public List<EsquemaCorrida> buscar(Marca marca, RolOperativo rolOperativo,
			Ruta ruta, ClaseServicio claseServicio) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		c.add(Restrictions.eq("ruta", ruta));
		c.add(Restrictions.eq("marca", marca));
		c.add(Restrictions.eq("claseServicio", claseServicio));
		c.add(Restrictions.eq("rolOperativo", rolOperativo));

		return c.list();
	}

	public List<EsquemaCorrida> buscar(EsquemaCorrida ec) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		// if (ec.getHorasalida() != null) {
		// c.add(Restrictions.eq("horasalida", ec.getHorasalida()));
		// }

		// c.add(Restrictions.eq("esquemaOperacional.fecfinvigencia",
		// ec.getEsquemaOperacional().getFecfinvigencia()));
		// c.add(Restrictions.eq("esquemaOperacional.fecniciovigencia",
		// ec.getEsquemaOperacional().getFecniciovigencia()));

		c.createCriteria("esquemaOperacional").add(Restrictions.and(Restrictions.eq("fecfinvigencia", ec.getEsquemaOperacional().getFecfinvigencia()),
				Restrictions.eq("fecniciovigencia", ec.getEsquemaOperacional().getFecniciovigencia())));

		// c.createCriteria("esquemaOperacional").add(Restrictions.eq("fecniciovigencia",
		// ec.getEsquemaOperacional().getFecniciovigencia()));

		if (ec.getIndlunes() != null) {
			c.add(Restrictions.eq("indlunes", ec.getIndlunes()));
		}

		if (ec.getIndmartes() != null) {
			c.add(Restrictions.eq("indmartes", ec.getIndmartes()));
		}

		if (ec.getIndmiercoles() != null) {
			c.add(Restrictions.eq("indmiercoles", ec.getIndmiercoles()));
		}

		if (ec.getIndjueves() != null) {
			c.add(Restrictions.eq("indjueves", ec.getIndjueves()));
		}

		if (ec.getIndviernes() != null) {
			c.add(Restrictions.eq("indviernes", ec.getIndviernes()));
		}

		if (ec.getIndsabado() != null) {
			c.add(Restrictions.eq("indsabado", ec.getIndsabado()));
		}

		if (ec.getInddomingo() != null) {
			c.add(Restrictions.eq("inddomingo", ec.getInddomingo()));
		}

		if (ec.getCantparados() != null) {
			c.add(Restrictions.eq("cantparados", ec.getCantparados()));
		}

		if (ec.getStatusCorrida() != null) {
			c.add(Restrictions.eq("statusCorrida", ec.getStatusCorrida()));
		}

		if (ec.getRuta() != null) {
			c.add(Restrictions.eq("ruta", ec.getRuta()));
		}

		if (ec.getRolOperativo() != null) {
			c.add(Restrictions.eq("rolOperativo", ec.getRolOperativo()));
		}

		if (ec.getMarca() != null) {
			c.add(Restrictions.eq("marca", ec.getMarca()));
		}

		// c.add(Restrictions.eq("esquemaOperacional",
		// ec.getEsquemaOperacional()));

		if (ec.getEmpresa() != null) {
			c.add(Restrictions.eq("empresa", ec.getEmpresa()));
		}

		if (ec.getEmpresa1() != null) {
			c.add(Restrictions.eq("empresa1", ec.getEmpresa1()));
		}

		if (ec.getClaseServicio() != null) {
			c.add(Restrictions.eq("claseServicio", ec.getClaseServicio()));
		}

		if (ec.getTipocorrida() != null) {
			c.add(Restrictions.eq("tipocorrida", ec.getTipocorrida()));
		}

		String cs = c.toString();

		return c.list();
	}

	public boolean esHijoRebote(EsquemaCorrida esquemaCorrida) {
		Criteria c = makeCriteria();
		c.setProjection(Projections.rowCount());
		c.add(Restrictions.eq("activo", Boolean.TRUE));
		c.add(Restrictions.eq("esquemaCorridaRebote", esquemaCorrida));

		Long cant = HibernateFix.count(c.list());

		return (cant > 0);
	}

	@Override
	public List<EsquemaCorrida> buscaDentroVigencia(Date dataGeracao) {
		Criteria c = makeCriteria();
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		Criteria cEsquemaOperacional = c.createCriteria("esquemaOperacional");
		cEsquemaOperacional.add(Restrictions.le("fecniciovigencia", dataGeracao));
		cEsquemaOperacional.add(Restrictions.ge("fecfinvigencia", dataGeracao));
		cEsquemaOperacional.add(Restrictions.eq("activo", Boolean.TRUE));

		return c.list();
	}

	public boolean buscarEsquemaCorridaExisteTramo(Tramo tramo) {
		Criteria c = makeCriteria();
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		Criteria cEsquemaOperacional = c.createCriteria("esquemaTramoList");
		cEsquemaOperacional.add(Restrictions.eq("tramo", tramo));

		if (c.list().isEmpty()) {
			return false;
		} else {
			return true;
		}
	}

	public List<Integer> existe(Ruta ruta, Integer paradaId) {

		Criteria c = makeCriteria();
		c.add(Restrictions.eq("ruta", ruta));
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		Criteria cEsquemaTramo = c.createCriteria("esquemaTramoList");
		cEsquemaTramo.add(Restrictions.eq("activo", Boolean.TRUE));

		Criteria cTramo = cEsquemaTramo.createCriteria("tramo");

		Disjunction or = Restrictions.disjunction();
		or.add(Restrictions.eq("origem.paradaId", paradaId));
		or.add(Restrictions.eq("destino.paradaId", paradaId));

		cTramo.add(or);

		c.setProjection(Projections.property("esquemacorridaId"));

		return c.list();
	}
}