package com.rjconsultores.ventaboletos.dao.hibernate;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
import com.rjconsultores.ventaboletos.dao.GenericDAO;
import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

/**
 * 
 * @author gleimar
 */
public class GenericHibernateDAO<T, ID extends Serializable> extends HibernateDaoSupport implements GenericDAO<T, ID> {

	@SuppressWarnings("unchecked")
	public GenericHibernateDAO() {
		this.persistentClass = (Class<T>) ((ParameterizedType) getClass().getGenericSuperclass()).getActualTypeArguments()[0];
	}

	// Classe que ser persistida.
	private Class<T> persistentClass;

	public Class<T> getPersistentClass() {
		return this.persistentClass;
	}

	@Override
	public void borrar(T entity) {
		try {
			this.getHibernateTemplate().delete(entity);
		} catch (final HibernateException ex) {
			throw convertHibernateAccessException(ex);
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public T obtenerID(ID id) {
		try {
			return (T) this.getHibernateTemplate().get(getPersistentClass(), id);
		} catch (final HibernateException ex) {
			throw convertHibernateAccessException(ex);
		}
	}

	@Override
	public List<T> obtenerTodos() {
		try {
			return this.getHibernateTemplate().loadAll(getPersistentClass());
		} catch (final HibernateException ex) {

			throw convertHibernateAccessException(ex);
		}
	}

	@Override
	public T suscribir(T entity) {
		try {
			this.getHibernateTemplate().save(entity);
			return entity;
		} catch (final HibernateException ex) {

			throw convertHibernateAccessException(ex);
		}
	}

	protected List<T> findByCriteria(Criterion... criterion) {
		try {
			Criteria crit = this.getHibernateTemplate().getSessionFactory().getCurrentSession().createCriteria(getPersistentClass());
			for (Criterion c : criterion) {
				crit.add(c);
			}
			return crit.list();
		} catch (final HibernateException ex) {
			throw convertHibernateAccessException(ex);
		}
	}

	protected Criteria makeCriteria() {
		return this.getHibernateTemplate().getSessionFactory().getCurrentSession().createCriteria(getPersistentClass());
	}

	public T actualizacion(T entity) {
		return getHibernateTemplate().merge(entity);
	}

	public Long count(String campo, Object o) {
		Criteria c = this.makeCriteria();

		c.add(Restrictions.eq(campo, o));
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		c.setProjection(Projections.rowCount());

		return HibernateFix.count(c.list());

	}
}
